/** Eve Manager user interface component class
 * 
 * @class UIComponent
 *
 *
 * @author <a href="mailto:info@ilevia.com">info@ilevia.com</a>
 *
 */
"use strict";

function UIComponent(){
	var self = this;
	
	/** Reference data
	 * @private 
	 * @type object
	 * @default undefined
	 */
	var m_refComponent = undefined;
	
	/** UI Component parameters
	 * @private 
	 * @type object
	 * @default new Object()
	 */
	var m_oParameters = new Object();
	
	/** UI Component type
	 * @private
	 * @type string
	 * @default undefined
	 */
	var m_strType = undefined;
	
	/** UIComponent - UIComponents Unique Id generator
     * @private
     * @type EveIdAssign
     */
    var m_oUIComponentsIdAssign = new EveIdAssign();

    /** UI Component - UIComponents level
     * @private
     * @type object
     * @default new Node();
     */
    //var m_oUIComponentsTree = new Node({ name: "UIComponents root" });
	
	/** UIComponent - UIComponents level id generator
     * @private
     * @type {undefined|Indexer}
     */
	//var m_oInterfaceUIIdAssign;
	
	/** Selected UI component
     * @private
     * @type Node
     * @default undefined
     */
    //var m_refSelUIComponent = undefined;
	
	
	/** UI Component maps
	 * @private
	 * @type object
	 * @default new Node();
	 */
	var m_oMapTree = new Node({ name: "Maps root" });
	
	/** UIComponent map id generator
	 * @public
	 * @type EveIdAssign
	 */
	var m_oMapIdAssign = new EveIdAssign();
	
	/** Selected map
     * @private
     * @type Node
     * @default undefined
     */
    var m_refSelMap = undefined;
	

	/** Initialization private function
	 *
	 */
	var init = function(){
        self.addParamVal("icon",new Icon());
        self.addParamVal("name",new String());
    };
	
	/** UI Component html id
	 * @public 
	 * @type string
	 * @default undefined
	 */
	this.id = undefined;
	
	/** UI Component Unique Id (maybe SID of Luca's Eve Manager version)
	 * @public
	 * @type Integer
	 * @default undefined
	 */
	this.UId = undefined;
	
	/** UI Component parent Unique Id
     * @private 
     * @type Integer
     * @default undefined
     */
    this.parentUId = undefined;
    
    		/** UI Level
     * @private
     * @type String
     * @default undefined
     */
    this.UILevel = undefined;


    
    /** UI Component's Component reference Unique Id
     * @private 
     * @type Integer
     * @default undefined
     */
    this.referenceUId = undefined;
	
	/** Set reference data
	 * @param {object} oData Reference data
	 */
	this.setRefComponent = function(oData){
		m_refComponent = oData;
	};
	
	/** Get reference data
	 * @returns {object} oData Reference data
	 */
	this.getRefComponent = function(oData){
		return m_refComponent;
	};
	
	/** Add new parameter and set value - Register action
	 * @param {string} key Key of memory area
	 * @param {string|Number} value of m_Data[Key]
	 * @returns {Integer} Error description (0 = Ok; otherwise = Error)
	 * @public
	 */
	this.addParamVal = function(key,value){
		//Check if key already exist
		if(m_oParameters[key] != undefined){ console.error("Parameter "+ key +" already exist"); return 1; }
		value = value || undefined;
		
		m_oParameters[key] = value;
		return 0;
	};
	
	/** Set parameter value
	 * @param {string} key Key of m_oParameters
	 * @param {string|Number} value of m_Data[Key]
	 * @returns {Integer} Error description (0 = Ok; otherwise = Error)
	 * @public
	 */
	this.setParamVal = function(key,value){
		//Check if key already exist []
		if(m_oParameters[key] == undefined){
		    console.error("Parameter "+ key +" do not exist");
		    return 1;
		}
		
		m_oParameters[key] = value;
		return 0;
	};
	
	/** Get parameter value
	 * @param {string} key Key of m_oParameters
	 * @returns {Integer} Error description (0 = Ok; otherwise = Error)
	 * @public
	 */
	this.getParamVal = function(key){
		return m_oParameters[key];
	};
	
	/** Set component type
	 * @param str type of component
	 * @returns {Integer} Error description (0 = Ok; otherwise = Error)
	 * @public
	 */
	this.setType = function(strType){
		//Check input parameter is a string
		if(typeof strType != "string"){
			console.error("Type of input parameter is not a string");
			return 1;
		}
		
		//Check if type member is already defined (can be defined only once)
		if(m_strType != undefined){
			console.error("Type parameter is already defined");
			return 2 ;
		}
		
		m_strType = strType;
		return 0;
	};
	
	/** Get component type
	 * @returns {string} Type of component
	 * @public
	 */
	this.getType = function(){
		return m_strType;
	};
	
	/** Get UI components reference
     * @returns {object} Maps reference object
     * @public
     */
    /*this.getUIComponents = function(){
        return m_oUIComponentsTree;
    };*/
	
	/** Get component maps reference
     * @returns {object} Maps reference object
     * @public
     */
    this.getMaps = function(){
        return m_oMapTree;
    };
	
	/** Add map to UIComponent
     * @returns {object} Maps New created map
     * @public
     */
    this.addMap = function(){
        var oMap = new Map();
        var oMapNode = new Node(oMap);
        var strMapId = self.id +"_map_"+ m_oMapIdAssign.getId();
        
        //Set map id
        oMap.id = strMapId;
        
        oMapNode.setData(oMap);
        m_oMapTree.addChild(oMapNode);
        return oMapNode;
    };
	
	/** Remove map to UIComponent
	 * @returns {object} Maps New created map
	 * @public
	 */
	this.removeMap = function(oNode){
	    //Check input oNode existance
        if(oNode == undefined){
            console.error("Specify an oNode to remove map");
            return 1;
        }
        
        //Check if oNode contain UIComponent            
        if(!oNode.getData() instanceof Map){
            console.error("oNode not contains Map instance");
            return 2;
        }
        
        var oMap = oNode.getData();
		var strMapId = oMap.id;
		
		//Search and remove
		//Check if UIComponent contain current Map
        var bFind = false;
        for(var i=0; i<m_oMapTree.children.length; ++i){
            
            if(m_oMapTree.children[i].getData() === oMap){
               
               //Check if is selected
               if(m_refSelMap === oNode){
                   m_refSelMap = undefined;
               }
               
               //Remove from array
               m_oMapTree.children.splice(i,1);
               
               bFind = true;
               break;
            }
        }
        if(!bFind){
            console.error("oMap is not a Map of this component");
            return 3;
        }
        
		
		return 0;
	};
	
	/** Set selected map
	 * @param oNode node object containing Map
	 * @returns {object} Maps New created map
	 * @public
	 */
	this.setSelectMap = function(oNode){
	    //Check Node instance
        if(oNode == undefined || !oNode instanceof Node){
            console.error("oNode is not a Node instance");
            return undefined;
        }
        
        var oMap = oNode.getData();
        
        //Check Map instance
        if(!oMap instanceof Map){
            console.error("oMap is not a Map instance");
            return undefined;
        }
        
        //Check if UIComponent contain current Map
        var bFind = false;
        for(var i=0; i<m_oMapTree.children.length; ++i){
            
            if(m_oMapTree.children[i].getData() === oMap){
               bFind = true;
               break;
            }
        }
        if(!bFind){
            console.error("oMap is not a Map of this component");
            return undefined;
        }
        
        //Set selected map
        m_refSelMap = oNode;
        
		return m_refSelMap;
	};
	
	/** Get selected map
	 * @returns {object} Maps New created map
	 * @public
	 */
	this.getSelectMap = function(){
		return m_refSelMap;
	};
	
	/** Remove selection from selected map
	 * @public
	 */
	this.deselectMap = function(){
	    var oNode = m_refSelMap;
		m_refSelMap = undefined;
		return oNode;
	};
	
	/** Set component maps reference
     * @param object Maps reference object
     * @returns {Integer} Error description (0 = Ok; otherwise = Error)
     * @public
     */
    this.setMaps = function(oMaps){
        alert("Substitute this.setMaps with this.setMapsTree!!!!");
        //Check if type member is already defined (can be defined only once)
        /*if(m_oMapTree != undefined){
            console.error("Maps parameter is already defined");
            return 2 ;
        }*/
        
        m_oMapTree = oMaps;
        return 0;
    };
	
	/** Set Unique id generator
     * @returns {boolean}
	 */
    /*this.setUIdGenerator = function(oIndexer){
        
        //Check
        if(! (oIndexer instanceof Indexer)){
            console.error("oIndexer is not a Node Indexer");
            return false;
        }
        
        //Set UId generator
        m_oInterfaceUIIdAssign = oIndexer;
        
        return true;
    };*/
	
	/** Get component parameters (dump)
	 * @returns {object} Component parameters
	 * @public
	 */
	this.getParameters = function(){
		return m_oParameters;
	};
	
	/** JSON serialization
     *
     */
    this.encodeJSON = function(){
        
        //Serialize instance variables
        var oSerialized = {
            UId: self.UId,
            parentUid: self.parentUId,
            UILevel: self.UILevel,
            //type: m_strType,
            //refComponent: (m_refComponent!=undefined)?m_refComponent.getData().UId:undefined,
            referenceUId: self.referenceUId,
            maps: new Array()
        };
        
        //Serialize parameters
        for(var k in m_oParameters){
            oSerialized[k] = m_oParameters[k];
        }
        
        //Serialize maps
        for(var i=0; i<m_oMapTree.children.length; ++i){
            oSerialized.maps.push(m_oMapTree.children[i].encodeJSON());
            //console.warn(m_oMapTree.children[i].encodeJSON());
        }
        
        return oSerialized;
    };
	
	/** Overwrite toString() method
     * @returns {string} String that describe UIComponent object
     */
    this.toString = function(){
    	//Check linked node
    	if(m_refComponent != undefined && m_refComponent.getData() != undefined){
    		return 	"UIComponent - User Interface ID: "+ self.UId +
    				"; name: "+ m_oParameters["name"] +"; type: "+ m_strType +
    				"; linked with: "+ m_refComponent.toString();
    	}
    	else{
    		return "UIComponent - User Interface ID: "+ self.UId +"; name: "+ m_oParameters["name"] +"; type: "+ m_strType;
    	}
    };

    init();
}