﻿/**
 * @class User
 *
 * @description Eve Manager User Class
 *
 *
 * @author <a href="mailto:info@ilevia.com">info@ilevia.com</a>
 *
 */
"use strict";

function User(){
	//Instance variables (private)
	var self = this;

    /** User permissions
     * @private
     * @type Array
     * @default new Array()
     */

    var m_oPermissions = new Object();
	
	/** Initialization private function
	 * @private
	 */
    var init = function () {
        // Set Default logout Icon

        var panw=model["instance"].skin.IconsPan.imgWidth;
        var panh=model["instance"].skin.IconsPan.imgHeight;
        var iconw=model["instance"].skin.IconsPan.iconsWidth;
        var iconh=model["instance"].skin.IconsPan.iconsHeight;

        var i0=model["instance"].skin.ComponentsInfo.Logout.defaultIcons[0];
        var oDefaultPan = application.ResoucesManager.getDefaultIconsPan();
        self.logoutIcon.position.x = -i0.position.x*iconw;
        self.logoutIcon.position.y = -i0.position.y*iconh ;
        self.logoutIcon.status=0;
        self.logoutIcon.enableColors=false;
        self.logoutIcon.srcId= oDefaultPan["id"];
        self.logoutIcon.size.h=iconh;
        self.logoutIcon.size.w=iconw ;
        self.logoutIcon.setSize.h= panh;
        self.logoutIcon.setSize.w= panw;



        //TODO Fare un check. Se l'utente è admin aggiungere una funzione di update di nuovi (eventuali) permessi
    };
	
	/** User unique Name
     * @type string
     * @default undefined
     */
    this.name = undefined;

    /** User description
     * @type string
     * @default ""
     */
    this.description = "";
    
    /** User password
     * @private 
     * @type Mixed
     * @default undefined
     */
    this.password = undefined;

    /** User parent
     * @type string
     * @default undefined
     */
    this.createdBy = undefined;
    
    /** User Interface
     * @private
     * @type Array
     * @default new Array()
     */
    this.UI = new Array();

    /** XML User Interface
     */
    this.UIxml = "";

    /** Childrens Users
     */
    this.childsUsers = new Array();

    /** Created By Users
     */
    this.createdBy;

	/** User canvas dimension (iPad 2 dimension for default)
     * @private
     * @type object
     * @default { w: 1024, h: 768 }
     */
    this.canvas = { w: 1024, h: 768 };

    /** Top bar
     * @private
     * @type Number
     * @default
     */
    this.leftBar = 153;

    /** Flip left bar
     *
     */
    this.flipLeftBar = "false";

    /** Top bar
     * @private
     * @type Number
     * @default
     */
    this.topBar = 115;

    /** Flip top bar
     *
     */
    this.flipTopBar = "false";

    /** Background image
     * @private
     * @type Number
     * @default
     */
    this.backgroundImage = "";

    /** Logout icon
     * @private
     * @type Icon
     * @default
     */
    this.logoutIcon = new Icon();

    this.barColor = "000000";
    this.barAlpha = "80";
    this.barBlur = "true";




    /** Methods */

    /** Add permission
     *
     */
    this.addPermission = function(key,value){
        //Check if key already exist
        if(m_oPermissions[key] != undefined){
            return 1;
        }

        m_oPermissions[key] = value;
        return 0;
    };

    /** Set permission
     *
     */
    this.setPermission = function(key,value){
        // Check if key already exist []
        if(m_oPermissions[key] == undefined){
            return 1;
        }

        m_oPermissions[key] = value;
        return 0;
    };

    /** Get permission
     *
     */
    this.getPermission = function(key){

        //return m_oPermissions[key];
        return (m_oPermissions[key] != undefined);
    };

    /** Get all permissions
     *
     */
    this.getPermissions = function(){
        return m_oPermissions;
    };

    /** Reset all permissions
     *
     */
    this.resetPermissions = function(){
        if(self.name == "admin"){
            return 1;
        }

        m_oPermissions = new Object();
        return 0;
    };

    /** Set xml
     *
     */
    this.setUIxml = function(xml){
        if(typeof xml != "string"){
            throw "XML text expected";
        }

        self.UIxml = xml;
    };
	
	/** JSON serialization
	 *
	 */
	this.encodeJSON = function(){
        
        //Set instance variables
        var oSerialized = {
            //Name field must be unique
            name: self.name,
            description: self.description,
            password: self.password,
            logoutIcon: self.logoutIcon,
            //Set by application (eg. iPad, iPhone)
            canvas: self.canvas,
            leftBar: this.leftBar,
            flipLeftBar: this.flipLeftBar,
            flipTopBar: this.flipTopBar,
            topBar: this.topBar,

            barColor: this.barColor,
            barAlpha: this.barAlpha,
            barBlur: this.barBlur,


            UI: self.UI,
            UIxml: self.UIxml,
            permissions: m_oPermissions,
            backgroundImage: self.backgroundImage
            //childsUsers: self.childsUsers
        };

        if(self.createdBy != undefined){
            oSerialized.createdBy = self.createdBy.getData().name;
        }
        
        return oSerialized;
    };
	
	/** Overwrite toString() method
     * @returns {string} String that describe User object
     */
    this.toString = function(){
        return "[User] - User name: "+ self.name + ";";
    };

    //Initial operations
    init();
};