/** Loader class (extends Window class)
 *
 * @class loader
 *
 *
 * @author <a href="mailto:info@ilevia.com">info@ilevia.com</a>
 *
 * 
 * @required <a href="http://jquery.com/">jQuery</a>
 * @required <a href="http://jqueryui.com/">jQuery UI</a>
 * 
 */
"use strict";

function loader(oWindow){
	
	var self = this;
	
	/**
	 *
	 */
	var init = function(){
        console.log("%cWindow - loader class initialization (extends Window class)","color:green");
        
        //Create jQuery UI dialog
        m_$element.dialog({
            closeOnEscape: false,
            height: "auto",
            width: 500,
            modal: true,
            resizable: false,
            autoOpen: false,
            //close: m_oWindow.hide,
            close: self.onHide,
            create: function(e,ui){
                var $this = $(this);

                $this.find("button").button();
                
                //Remove header
                $this.parent().find(".ui-widget-header").hide();
                $this.parent().css("border-radius","0");
                $(".ui-widget-overlay").bind("click",function(){
                    $this.dialog("close");
                });
                
                //Create progress bar
                $this.find(".progressbar").progressbar({
                    value: false,
                    change: function(){
                        $(this).find(".progresslabel").text( parseInt( $(this).progressbar("value"),10 ) + "%" );
                    },
                    complete: function() {
                        $(this).find(".progresslabel").text( _D("Complete") );
                    }
                });
            }
        });

        //On enable/disable color
        m_$element.on("click",".cancel",function(){
            QT.abortLoading();
            m_oWindow.hide();
            //m_$element.dialog("close");
        });
        
        //Hide loader type
        m_$element.find(".loader-type").hide();
        
        //Check activity
        while(!m_queActivity.isEmpty()){
            var o = m_queActivity.dequeue();
            
            switch(o.action){
                case "show":
                    self.show(o.parameters);
                    break;
                case "hide":
                    m_oWindow.hide();
                    break;
            }
        }
    };
	
	/** Extended window object 
     * @type {Window}
     */
	var m_oWindow = oWindow;
	
	/** jQuery DOM element representing Window GUI
     * @type {jQuery}
     */
    var m_$element;
    
    /** Loading counter
     * @type {Integer}
     */
    var m_iLoadingCounter = 0;
    
    /** Loading timeout
     * @type {setTimeout}
     */
    var m_queTimeOut = new bigh.Queue();
    
    /** Loading timeout
     * @type {setTimeout}
     */
    var m_oTimeOut;
    
    /** Loading timeout (2 min)
     * @type {Integer}
     */
    var m_iTimeOut = 120000;
    
    /** Loading timeout function
     * @type {Function}
     */
    var m_fTimeOutFunc = function(){
        console.error("Loading close with timeout");
        
        m_bStatus = false;
        
        //Close window
        m_oWindow.hide();
    };
    
    /** Loader status (true = OK; false = ERROR)
     * @type {Boolean}
     */
    var m_bStatus;
    
    /** Activity before DOM loaded
     * @type {setTimeout}
     */
    var m_queActivity = new bigh.Queue();
	
	/** Show icon editor window
	 */
	this.show = function(oParam){
	    
	    //Check active
	    if(m_$element == undefined){
	        m_queActivity.enqueue({
	            action: "show",
	            parameters: oParam
	        });
	        return;
	    }
	    
	    var oDefault = {
	        //Accepted value: "default", "bar"
	        type: "default",
	        //Accepted value: string and HTML string
	        message: "Wait"
	    };
	    
	    //Extend default object
	    $.extend(true,oDefault,oParam);
	    
	    //Hide all
	    m_$element.find(".loader-type").hide();
	    m_$element.find(".ani-gif").hide();
	    m_$element.find(".progressbar").progressbar("value",0);
	    
	    //Switch loader type
	    switch(oDefault.type){
	        case "default":
	           m_$element.find(".default").show();
	           m_$element.find(".ani-gif").show();
	           break;
	        case "bar":
	           m_$element.find(".bar").show();
               break;
	    }
	    
	    //Print message
	    m_$element.find(".message").html(oDefault.message);
	    
	    //Increment loader counter
	    m_iLoadingCounter++;
	    
	    //Set timeout
	    m_queTimeOut.enqueue( setTimeout(m_fTimeOutFunc,m_iTimeOut) );
	    
	    m_$element.dialog("open");
	};
	
	/** Hide icon editor window and closing operation
     */
    this.hide = function(e,ui){
        //Check active
        if(m_$element == undefined){
            m_queActivity.enqueue({
                action: "hide"
            });
            return;
        }
        
        //Check status
        /*if(!m_bStatus){
            console.error("Loading close with timeout during: \""+ m_$element.find(".message").text() +"\"");
            m_bStatus = true;
        }*/
        
        //Decrement loader counter
        m_iLoadingCounter--;
        
        //Reset close timeout
        clearTimeout(m_queTimeOut.dequeue());
        
        if(m_iLoadingCounter < 1){
            //Close window
            m_$element.dialog("close");
            
            //Check counter
            if(m_iLoadingCounter < 0){
                m_iLoadingCounter = 0;
            }
        }
    };
	
	/** Hide icon editor window and closing operation
	 */
	this.onHide = function(e,ui){
	    //console.error("onHide");
	    return;
	    
	    //Decrement loader counter
        m_iLoadingCounter--;
        
        //Reset close timeout
        //clearTimeout(m_oTimeOut);
        var temp = m_queTimeOut.dequeue();
        clearTimeout(temp);
	    
	    //Close
	    if(m_iLoadingCounter < 1){
	        
	        
	        
	        
	    }
	};
	
	/** getJQuery element 
     * @param $element jQuery element represent this window
     * @returns {Node}
     */
    this.get$elm = function($element){
        return m_$element;
    };
	
	/** Assign window HTML element 
     * @param $element jQuery element represent this window
     * @returns {Node}
     */
    this.assignWindow = function($element){
        m_$element = $element;
        
        //Initialization after graphic ready
        init();
    };
    
}