/** User Manager class (extends Window class)
 *
 * @class userManager
 *
 *
 * @author <a href="mailto:info@ilevia.com">info@ilevia.com</a>
 *
 */
"use strict";

function userManager(oWindow){
	
	var self = this;
	
	/**
	 *
	 */
	var init = function(){
        console.log("%cWindow - userManager class initialization (extends Window class)","color:green");
        
        //Create jQuery UI dialog
        m_$element.dialog({
            title: _D("User management"),
            height: "auto",
            width: "auto",
            minWidth: 360,
            resizable: false,
            autoOpen: false,
            modal: true,
            close: m_oWindow.hide,
            create: function(e,ui){
                $(this).find(".actions button").button();
                $(this).find(".selectImage").button();
            }
        });

        //On select image
        m_$element.on("click",".selectImage",self._selectImage);
        
        //On remove image
        m_$element.on("click",".removeImage",self._removeImage);
        
        //On choose icon
        m_$element.on("click",".icon",self._editIcon);
        
        //On Submit
        m_$element.on("submit","form",self._submit);
        
        //On reset
        m_$element.on("click",".actions .cancel",function(e){
            m_oWindow.hide();
        });

        //On duplicate
        m_$element.on("click",".actions .duplicate",self._duplicate);
        
        //On remove
        m_$element.on("click",".actions .remove",self._removeComponent);

        //On choose icon
        m_$element.on("click",".logoutIcon",self._editIcon);


        
    };
	
	/** Extended window object 
     * @type {Window}
     */
	var m_oWindow = oWindow;
	
	/** jQuery DOM element representing Window GUI
     * @type {jQuery}
     */
    var m_$element;
    
    /** Additional parameters
     * @type {undefined}
     */
    var m_oParam = undefined;

    /** Show icon editor window
     * @param oParam
     */
	this.show = function(oParam){
        if(m_$element == undefined || m_$element.length < 1){
            console.error("Window object databaseManager not be initialized");
            return;
        }

	    //Set parameters
	    m_oParam = oParam;

        var $permissionsList = m_$element.find(".permissions-list");
        var oPermissions = model["connectedUser"].getData().getPermissions();

        //Print avaiable permissions from Connected User
        //console.error(oPermissions);
        for(var k in oPermissions){
            var $element = getPermissionHtmlElement();

            //$element.find(".permission-id").attr("permission-id", k).prop("checked",oPermissions[k]["enabled"]);
            $element.find("input[type='checkbox']").attr("permission-id", k);
            $element.find(".label").attr("permission-label", oPermissions[k]).text(_D( oPermissions[k] ));

            $permissionsList.append($element);
        }

        //Get active user
        var oNode = m_oWindow.getActiveNode();

        if(oNode != undefined){
            var oUser = oNode.getData();
            var oActivePermissions = oUser.getPermissions();

            //Select active permissions
            for(var k in oActivePermissions){
                m_$element.find("[permission-id='"+ k +"']").prop("checked",true);
            }
        }

        if(m_oParam == "edit"){
            //var oNode = m_oWindow.getActiveNode();
            //var oUser = oNode.getData();

            m_$element.find(".name").val(oUser.name);
            m_$element.find(".description").val(oUser.description);

            m_$element.find(".canvas-width").val(oUser.canvas.w);
            m_$element.find(".canvas-height").val(oUser.canvas.h);

            m_$element.find(".left-bar").val(oUser.leftBar);
            m_$element.find(".top-bar").val(oUser.topBar);

            m_$element.find(".flip-vertical-bar").val(oUser.flipLeftBar).attr("selected", "selected");
            m_$element.find(".flip-horizontal-bar").val(oUser.flipTopBar).attr("selected", "selected");


            m_$element.find(".backgroundImage").val(oUser.backgroundImage);

            m_$element.find(".barColor").val(oUser.barColor);
            m_$element.find(".barAlpha").val(oUser.barAlpha);
            m_$element.find(".barBlur").prop("checked",oUser.barBlur);



            m_$element.find("#barChangeColor").css("background-color","#"+oUser.barColor);

            //Logout icon
            var $icon = m_$element.find(".logoutIcon");
            var $li = $icon.parents("li");

            //var $icon = m_$element.find(".icon");
            var $canvas = $("<canvas></canvas>");
            oUser.logoutIcon.srcId="app/resource/0";
            var oPan = application.ResoucesManager.getIconsPanById(oUser.logoutIcon.srcId);
            var img = document.getElementById(oUser.logoutIcon.srcId);
            //var img = document.getElementById("./css/icons-set/images/icons-state_1-128x128.png");

            //console.error(oUser.logoutIcon);

            //Default image loading
             var imgWidth = parseInt(oPan["params"]["imgWidth"],10);                          // Image width
            var imgHeight = parseInt(oPan["params"]["imgHeight"],10);
            var stretchScale = 0.5;
            var drawHeight = imgHeight * stretchScale;
            var drawWidth = imgWidth * stretchScale;
            var canvas = $canvas[0];
            var cont = canvas.getContext("2d");

            $canvas.attr("height", oPan["params"]["iconsHeight"] * stretchScale);
            $canvas.attr("width", oPan["params"]["iconsWidth"] * stretchScale);

            //Height - Width streched
            cont.drawImage(img,(oUser.logoutIcon.position.x * stretchScale),(oUser.logoutIcon.position.y * stretchScale),drawWidth,drawHeight);

            //var imageData = cont.getImageData(0,0,drawWidth,drawHeight);
            var imageData = cont.getImageData(0,0,64,64);

            //Color image
            if(oUser.logoutIcon.enableColors){
                //Color image
                for(var j=0; j<imageData.data.length; j+=4){
                    imageData.data[j+0] = oUser.logoutIcon.color.R;
                    imageData.data[j+1] = oUser.logoutIcon.color.G;
                    imageData.data[j+2] = oUser.logoutIcon.color.B;
                    imageData.data[j+3] = imageData.data[j+3];
                }
            }

            cont.putImageData(imageData,0,0);

            //Append Icon
            $icon.html($canvas);
            $icon.data({ eveManagerIcon: oUser.logoutIcon });

            //end draw icon

            if(oUser.name == model["connectedUser"].getData().name ){
                m_$element.find(".permissions").hide();                                     //Show permissions
                //m_$element.find(".actions .remove").button("option","disabled",true);
                m_$element.find(".actions .remove").hide();
            }

            if(oUser.name == "admin"){
                m_$element.find(".name").prop("disabled",true);                             //Show permissions
                m_$element.find(".permissions").hide();                                     //Show permissions
                //m_$element.find(".actions .remove").button("option","disabled",true);
                m_$element.find(".actions .remove").hide();
            }
        }
        else if(m_oParam == "new"){
            m_$element.find(".actions .remove").hide();
            m_$element.find(".actions .duplicate").hide();
            m_$element.find(".logoutIcon").parent().hide();

            m_$element.find(".canvas-width").val(1024);
            m_$element.find(".canvas-height").val(768);

            m_$element.find(".left-bar").val(153);
            m_$element.find(".top-bar").val(115);
            m_$element.find(".flip-vertical-bar").val("false").attr("selected", "selected");

            m_$element.find(".flip-horizontal-bar").val("false").attr("selected", "selected");

            m_$element.find(".barColor").val("000000");
            m_$element.find(".barAlpha").val("80");
            m_$element.find(".blur").val("true");
            m_$element.find("#barChangeColor").css("background-color","FFFFFF");


            //Flag default on use eve remote
            m_$element.find(".permissions [permission-id='21']").prop("checked",true);
        }
	    
	    m_$element.dialog("open");

        if( m_$element.find(".name").is("enabled")){
            m_$element.find(".name")[0].select();
        }
	};

    /** Hide icon editor window and closing operation
     * @param ui
     * @param e
     */
	this.hide = function(e,ui){
	    //Reset parameters
        m_oParam = undefined;

        m_$element.find(".name").removeProp("disabled");

        m_$element.find(".permissions").show();         //Show permissions
        m_$element.find("input[type='text'],textarea,input[type='password']").val("");  //Empty text fields
        m_$element.find(".error-msg").text("").hide();         //Empty errors
        m_$element.find(".permissions-list").empty();

        m_$element.find(".logoutIcon").parent().show();

	    //Show hidden elements
        m_$element.find(".actions .remove").button("option","disabled",false);
        m_$element.find(".actions .remove").show();
        m_$element.find(".actions .duplicate").show();
	    
	    //Close
	    m_$element.dialog("close");
	};
	
	/** Assign window HTML element 
     * @param $element jQuery element represent this window
     * @returns {Node}
     *
     */
    this.assignWindow = function($element){
        m_$element = $element;
        
        //Initialization after graphic ready
        init();
    };
    
    /** Interactions methods */

    /** Chose icon
     * @param e
     */
    this._editIcon = function(e){
        var oNode = m_oWindow.getActiveNode();
        
        //Call external window hide method (close current window)
        m_oWindow.hide();
        
        //Open window editor
        presenter.getWindow("iconEditor").show(oNode,{});
    };

    /** Duplicate user
     *
     * @param e
     */
    this._duplicate = function(e){
        var oNode = m_oWindow.getActiveNode();

        //Save current user
        application.saveUser(oNode);


        var oSerialization = oNode.encodeJSON();

        //console.error( oSerialization );

        //Check name
        var oRoot = model["instance"].getUsersRoot();
        var iCount = 1;
        var bFree = false;

        while(!bFree){
            bFree = true;
            for(var i=0; i<oRoot.children.length; ++i){

                if(oRoot.children[i].encodeJSON().name == oSerialization.name +"("+ iCount+")"){
                    iCount++;
                    bFree = false;
                    break;
                }
                //console.error(oRoot.children[i].encodeJSON().name);
            }
        }

        //console.error(oSerialization.name +"("+ iCount+")");
        oSerialization.name = oSerialization.name +"("+ iCount+")";
        var oNewNode = application.addUser(oSerialization,model["connectedUser"]);

        //console.error( oNewNode.encodeJSON() );
        application.selectUser(oNewNode);

        //Call external window hide method
        m_$element.dialog("close");

        //Open window editor
        //presenter.getWindow("userManager").show( oNewNode, "edit" );
    };

    /** Remove component
     *
     * @param e
     */
    this._removeComponent = function(e){
        var oNode = m_oWindow.getActiveNode();

        application.removeUser(oNode);
        
        //Call external window hide method
        m_$element.dialog("close");
    };

    /** Select image
     *
     * @param e
     */
    this._selectImage = function(e){
        //var oNode = m_oWindow.getActiveNode();
        //var oUser = oNode.getData();

        var path = QT.selectImage( parseInt(m_$element.find(".canvas-width").val(),10), parseInt(m_$element.find(".canvas-height").val(),10) );

        m_$element.find(".backgroundImage").val(path);

    };
    
    /** Remove image
     *
     * @param e
     */
    this._removeImage = function(e){
        //var oNode = m_oWindow.getActiveNode();
        //var oUser = oNode.getData();

        m_$element.find(".backgroundImage").val('');
    };

    /** Chose icon
     *
     */
    this._editIcon = function(e){

        /*var oIcon = $(this).data("eveManagerIcon");

         //Open window editor
         presenter.getWindow("iconEditor").show(oNode,{});
         return;*/
        var oNode = m_oWindow.getActiveNode();
        var oIcon = $(this).data("eveManagerIcon");

        self._submit();

        //Call external window hide method (close current window)
        m_oWindow.hide();

        //Open window editor
        presenter.getWindow("iconEditor").show(oNode,{ "iconstatus": oIcon.status });
    };

    /** Confirm changes
     *
     * @param e
     */
    this._submit = function(e){
        var newName = m_$element.find(".name").val();
        var newDescription = m_$element.find(".description").val();
        var newPassword = m_$element.find(".new-password").val();
        var confirmPassword = m_$element.find(".confirm-password").val();
        var checkPsw = false;

        var canvas = {
            h: m_$element.find(".canvas-height").val(),
            w: m_$element.find(".canvas-width").val()
        };
        
        var leftBar = m_$element.find(".left-bar").val();
        var flipLeftBar = m_$element.find(".flip-vertical-bar").val();

        var topBar = m_$element.find(".top-bar").val();
        var flipTopBar = m_$element.find(".flip-horizontal-bar").val();




        var backgroundImage = m_$element.find(".backgroundImage").val();

        var col=bigh.textRgb2hex(m_$element.find("#barChangeColor").css("background-color"));
        col=col.slice(1,7);
        var barColor = col;
        var barAlpha = m_$element.find(".barAlpha").val();
        var barBlur = m_$element.find(".barBlur").prop("checked");


        //Check operation
        if(m_oParam == "edit"){
            var oNode = m_oWindow.getActiveNode();
            var oUser = oNode.getData();

            if(newName.length < 1){
                console.error("Specify a user's name");
                m_$element.find(".error-msg").text(_D("Specify a user's name")).show();
                return;
            }

            if(newPassword.length > 0){
                if(newPassword != confirmPassword){
                    console.error("Passwords are different");
                    m_$element.find(".error-msg").text(_D("Passwords are different")).show();
                    return;
                }

                checkPsw = true;
            }

            //Set model and view
            $.publish("/model/user/"+ oUser.id +"/update/parameter",[oUser,"name",newName]);
            $.publish("/view/user/"+ oUser.id +"/refresh/parameter",[oUser,"name"]);

            if(checkPsw){
                $.publish("/model/user/"+ oUser.id +"/update/parameter",[oUser,"password", hex_md5(newPassword) ]);
                $.publish("/view/user/"+ oUser.id +"/refresh/parameter",[oUser,"password"]);
            }

            $.publish("/model/user/"+ oUser.id +"/update/parameter",[oUser,"description",newDescription]);
            $.publish("/view/user/"+ oUser.id +"/refresh/parameter",[oUser,"description"]);

            var oldCanvas = oUser["canvas"];

            $.publish("/model/user/"+ oUser.id +"/update/parameter",[oUser,"canvas",canvas]);
            //$.publish("/view/user/"+ oUser.id +"/refresh/parameter",[oUser,"canvas"]);

            $.publish("/model/user/"+ oUser.id +"/update/parameter",[oUser,"leftBar",leftBar]);
            $.publish("/model/user/"+ oUser.id +"/update/parameter",[oUser,"flipLeftBar",flipLeftBar]);
            //$.publish("/view/user/"+ oUser.id +"/refresh/parameter",[oUser,"description"]);

            $.publish("/model/user/"+ oUser.id +"/update/parameter",[oUser,"topBar",topBar]);
            $.publish("/model/user/"+ oUser.id +"/update/parameter",[oUser,"flipTopBar",flipTopBar]);
            //$.publish("/view/user/"+ oUser.id +"/refresh/parameter",[oUser,"description"]);

            $.publish("/model/user/"+ oUser.id +"/update/parameter",[oUser,"backgroundImage",backgroundImage]);

            $.publish("/model/user/"+ oUser.id +"/update/parameter",[oUser,"barColor",barColor]);
            $.publish("/model/user/"+ oUser.id +"/update/parameter",[oUser,"barAlpha",barAlpha]);
            $.publish("/model/user/"+ oUser.id +"/update/parameter",[oUser,"barBlur",barBlur]);


            //Set new permissions
            oUser.resetPermissions();
            var $activePermissions = m_$element.find(".permissions-list [permission-id]:checked");
            for(var i=0; i<$activePermissions.length; ++i){
                var $el = $( $activePermissions[i] );
                var id = $el.attr("permission-id");
                var label = $el.parent().find(".label").attr("permission-label");

                oUser.addPermission(id,label);
            }

            application.selectUser(oNode);

            //Fix icons
            if(canvas.w != oldCanvas.w || canvas.h != oldCanvas.h){
                var xScale = canvas.w/oldCanvas.w;
                var yScale = canvas.h/oldCanvas.h;

                for(var i=0; i<oUser.UI.length; ++i){
                    var oNode = application.getUIComponentByUid( oUser.UI[i].UId );
                    var oMaps = oNode.getData().getMaps();
                    //console.error(oNode);
                    //console.error(aMaps);

                    for(var j=0; j<oMaps.children.length; ++j){
                        //console.error( oMaps.children[j].getData() );
                        var aComponents = oMaps.children[j].getData().components;

                        for(var h=0; h<aComponents.length; ++h){
                            var oMapComponent = aComponents[h].getData();

                            //Change position
                            var oPosition = {
                                x: oMapComponent.position.x*xScale,
                                y: oMapComponent.position.y * yScale
                            };
                            $.publish("/model/uicomponent/map/mapcomponent/"+ oMapComponent.id +"/update/parameter",[oMapComponent,"position",oPosition]);
                            $.publish("/view/uicomponent/map/mapcomponent/"+ oMapComponent.id +"/refresh/parameter",[oMapComponent,"position"]);


                            //Check position
                            /*if((oMapComponent.position.x+oMapComponent.position.x)){

                            }
                            if(){}*/
                        }
                    }
                }
            }

            //TODO Remove permissions from childens
        }
        else if(m_oParam == "new"){
            if(newName.length < 1){
                console.error("Specify a user's name");
                m_$element.find(".error-msg").text(_D("Specify a user's name")).show();
                return;
            }

            if(newPassword.length < 1){
                console.error("Specify a user's password");
                m_$element.find(".error-msg").text(_D("Specify a user's password")).show();
                return;
            }

            if(newPassword != confirmPassword){
                console.error("Passwords are different");
                m_$element.find(".error-msg").text(_D("Passwords are different")).show();
                return;
            }

            try{
                var oUserNode = application.addUser({
                    name: newName,
                    password: hex_md5(newPassword),
                    description: newDescription,
                    canvas: canvas,
                    leftBar: leftBar,
                    topBar: topBar,
                    barColor: barColor,
                    barAlpha: barAlpha,
                    barBlur: barBlur,

                    backgroundImage: backgroundImage
                    //createdBy: model["connectedUser"].name
                },model["connectedUser"]);

                //Get user
                var oNewUser = oUserNode.getData();

                //console.warn(oNewUser.encodeJSON());

                //Set permission
                var $activePermissions = m_$element.find(".permissions-list [permission-id]:checked");
                for(var i=0; i<$activePermissions.length; ++i){
                    var $el = $( $activePermissions[i] );
                    var id = $el.attr("permission-id");
                    var label = $el.parent().find(".label").attr("permission-label");
                    oNewUser.addPermission(id,label)
                }

                application.selectUser(oUserNode);

                //console.warn(oNewUser.encodeJSON());
            }
            catch(err){
                console.error(err);
                m_$element.find(".error-msg").text(_D(err)).show();
                return;
            }

        }
        
        //Call external window hide method
        m_oWindow.hide();
    };

    /** Create permission html list item
     *
     */
    var getPermissionHtmlElement = function(){
        var $li = $("<li></li>");
        var $check = $("<input type='checkbox'></input>");
        var $label = $("<span style='margin-left: 20px;'></span>");

        //$check.addClass("permission");
        $label.addClass("label");
        $li.addClass("permission");

        $li.append($check);
        $li.append($label);

        return $li;
    };
}