﻿/** BigH Utilities
 *
 *
 *
 *
 * @required <a href="http://jquery.com/">jQuery</a>
 * @required <a href="http://jqueryui.com/">jQuery UI</a>
 */
"use strict";

var bigh = {
    /** Classes */
   
    /** Array stack */
    Stack: function(){
        var self = this;
        var stack = new Array();
         
        this.pop = function(){
           return stack.pop();
        };
        
        this.push = function(item){
           stack.push(item);
        };
        
        this.length = function(){
            return stack.length;
        };
        
        this.isEmpty = function(){
            return stack.length == 0;
        };
        
        this.empty = function(){
            while(stack.length > 0){
                self.pop();
            }
        };
    },
    /** Array queue */
    Queue: function(){
        var self = this;
        var queue = new Array();
        
        this.dequeue = function(){
           return queue.shift();
        };
        
        this.enqueue = function(item){
           queue.push(item);
        };
        
        this.length = function(){
            return queue.length;
        };
        
        this.isEmpty = function(){
            return queue.length == 0;
        };
         
        this.empty = function(){
            while(queue.length > 0){
                self.dequeue();
            }
        };
    },    
    /** Functions */
    
	/** Press only positive integer numbers
	 * 
	 * @param evt
	 */
	onlyPosInt: function(evt){
		var theEvent = evt || window.event;
		var key = theEvent.keyCode || theEvent.which;
		
		//Enter key
        if(key == 13){
            return;
        }
		
		key = String.fromCharCode(key);
		
		//var regex = /[0-9]|\./;
		var regex = /[0-9]/;
		//var regex = /[0-9]|\.|\-/;						//Add also minus char
		
		if(!regex.test(key)){
			theEvent.returnValue = false;
			if(theEvent.preventDefault) theEvent.preventDefault();
		}
	},
	/** Press only integer numbers (positive and negative)
	 * 
	 * @param evt
	 */
	onlyInt: function(evt){
		var theEvent = evt || window.event;
		var key = theEvent.keyCode || theEvent.which;
		
		//Enter key
        if(key == 13){
            return;
        }
		
		//console.log(theEvent.type);
		
		key = String.fromCharCode( key );
		
		var regex = /[0-9]|\-/;
		//var regex = /[0-9]|\.|\-/;						//Add also minus char
		
		if(!regex.test(key)){
			theEvent.returnValue = false;
			if(theEvent.preventDefault) theEvent.preventDefault();
		}
	},
	/** Press only float numbers
	 * 
	 * @param evt
	 */
	onlyFloat: function(evt){
		var theEvent = evt || window.event;
		var key = theEvent.keyCode || theEvent.which;
		
		//Enter key
        if(key == 13){
            return;
        }
        
		//console.log(theEvent.type);
		
		key = String.fromCharCode( key );
		
		//var regex = /[0-9]|\./;
		var regex = /[0-9]|\.|\-/;						//Add also minus char
		
		if(!regex.test(key)){
			theEvent.returnValue = false;
			if(theEvent.preventDefault) theEvent.preventDefault();
		}
	},
	/** Press only positive float numbers
     * 
     * @param evt
     */
    onlyPosFloat: function(evt){
        var theEvent = evt || window.event;
        var key = theEvent.keyCode || theEvent.which;
        
        //Enter key
        if(key == 13){
            return;
        }
        //console.log(theEvent.type);
        
        key = String.fromCharCode( key );
        
        //var regex = /[0-9]|\./;
        //var regex = /[0-9]|\.|\-/;                      //Add also minus char
        var regex = /[0-9]|\./;
        
        if(!regex.test(key)){
            theEvent.returnValue = false;
            if(theEvent.preventDefault) theEvent.preventDefault();
        }
    },
	/** Press only ip address
	 * 
	 * @param evt
	 */
	onlyIpAddress: function(elm){
		var theEvent = window.event;
		var key = theEvent.keyCode || theEvent.which;
		var $this = $(elm);
		var strValue = $this.val();
		var intRegex = /[0-9]|\./;
		
		//Enter key
        if(key == 13){
            return;
        }
		
		key = String.fromCharCode(key);
		
		if(!intRegex.test(key)){
			theEvent.returnValue = false;
			if(theEvent.preventDefault) theEvent.preventDefault();
		}
		else if(strValue.length >= 15){
			theEvent.returnValue = false;
			if(theEvent.preventDefault) theEvent.preventDefault();
		}
 	},
	/** Convert from RGB to HEX
 	 *
	 */
	textRgb2hex: function(rgb){
	    var hexDigits = new Array("0","1","2","3","4","5","6","7","8","9","a","b","c","d","e","f");
	    
	    rgb = rgb || "rgb (0, 0, 0)";
	    rgb = rgb.match(/^rgb\((\d+),\s*(\d+),\s*(\d+)\)$/);
	    
        function hex(x){
          return isNaN(x) ? "00" : hexDigits[(x - x % 16) / 16] + hexDigits[x % 16];
        }
        
        return "#" + hex(rgb[1]) + hex(rgb[2]) + hex(rgb[3]);
	},
	/** Convert from RGB to HEX
     *
     */
    rgb2hex: function(r,g,b){
        return "#" + ((1 << 24) + (r << 16) + (g << 8) + b).toString(16).slice(1);
    },
	/** Convert from HEX to RGB
     *
     */
    hex2rgb: function(hex){
        var result = /^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i.exec(hex);
        return result ? {
            r: parseInt(result[1], 16),
            g: parseInt(result[2], 16),
            b: parseInt(result[3], 16)
        } : null;
    },
    escapeHtml: function(text) {
        return text
            .replace(/&/g, "&amp;")
            .replace(/</g, "&lt;")
            .replace(/>/g, "&gt;")
            .replace(/"/g, "&quot;")
            .replace(/'/g, "&#039;");
    },
    decodeHtmlSpecialChar: function(text){
        return text
            //.replace("&amp;")
            .replace(/&lt;/gi,"<")
            .replace(/&gt;/gi,">");
    },
    cloneCanvas: function(oldCanvas){
        //create a new canvas
        var newCanvas = document.createElement('canvas');
        var context = newCanvas.getContext('2d');

        //set dimensions
        newCanvas.width = oldCanvas.width;
        newCanvas.height = oldCanvas.height;

        //apply the old canvas to the new one
        context.drawImage(oldCanvas, 0, 0);

        //return the new canvas
        return newCanvas;
    }
};

/** String utilities
 * 
 */

/** Capitalize first letter of a string
 * 
 * @returns {String}
 */
String.prototype.capitalizeFirst = function(){
    return this.charAt(0).toUpperCase() + this.slice(1);
};

Function.prototype.singleton = function(){
    console.error(this);
};

/* String.prototype.bigh = {
    capitalizeFirst: function(){
        //return this.charAt(0).toUpperCase() + this.slice(1);
        return this;
    }
}; */

/* String.bigh = {
    capitalizeFirst: function(){
        //return this.charAt(0).toUpperCase() + this.slice(1);
        return this;
    }
}; */

/* String.prototype.bigh.capitalizeFirst = function(){
    return this.charAt(0).toUpperCase() + this.slice(1);
    //return this;
}; */

//console.log( String.prototype.bigh.capitalizeFirst("ciccio") );
